@@ -18,7 +18,6 @@
 #include "Array.h"
 #include "Config.h"
 #include "Game.h"
-#include "Overlay.h"
 #include "Path.h"
 #include "Canvas.h"
 #include "Font.h"
@@ -27,6 +26,8 @@
 #include "Os.h"
 #include "Scene.h"
 #include "Script.h"
+#include "Dialogs.h"
+#include "Ui.h"
 
 #include <SDL/SDL.h>
 #include <SDL/SDL_image.h>
@@ -49,83 +50,52 @@ unsigned char levelbuf[64*1024];
 
 
 
-class CollectionSelector : public ListProvider
-{
-  Overlay* m_list;
-public:
-  CollectionSelector( GameControl& game )
-  {
-#if 0
-    m_list = createListOverlay( game, this );
-#endif
-  }
-  Overlay* overlay() { return m_list; }
-
-  virtual int     countItems() {
-    return 58;
-  }
-  virtual Canvas* provideItem( int i, Canvas* old ) {
-    delete old;
-    char buf[18];
-    sprintf(buf,"%d. Item",i);
-    Canvas* c = new Canvas( 100, 32 );
-    c->setBackground(0xffffff);
-    c->clear();
-    Font::headingFont()->drawLeft( c, Vec2(3,3), buf, i<<3 );
-    return c;
-  }
-  virtual void    releaseItem( Canvas* old ) {
-    delete old;
-  }
-  virtual void    onSelection( int i, int ix, int iy ) { 
-    printf("Selected: %d (%d,%d)\n",i,ix,iy);
-  }
-};
-
-
-class Game : public GameControl, public Widget
+class Game : public GameControl, public Container
 {
   Scene   	    m_scene;
   Stroke  	   *m_createStroke;
   Stroke           *m_moveStroke;
-  Array<Overlay*>   m_overlays;
-  Window            m_window;
-  Overlay          *m_pauseOverlay;
-  Overlay          *m_editOverlay;
-  Overlay          *m_completedOverlay;
-  CollectionSelector m_cselector;
+  Widget           *m_pauseLabel;
+  Widget           *m_editLabel;
+  Widget           *m_completedDialog;
+  Widget           *m_options;
   Os               *m_os;
   bool              m_isCompleted;
-  bool              m_paused;
   Path              m_jointCandidates;
   Path              m_jointInd;
 public:
   Game( Levels* levels, int width, int height ) 
   : m_createStroke(NULL),
     m_moveStroke(NULL),
-    m_window(width,height,"Numpty Physics","NPhysics"),
-    m_pauseOverlay( NULL ),
-    m_editOverlay( NULL ),
-    m_completedOverlay( createNextLevelOverlay(*this) ),
+    m_pauseLabel( NULL ),
+    m_editLabel( NULL ),
+    m_completedDialog( NULL ),
     m_isCompleted(false),
-    m_cselector( *this ),
+    m_options( NULL ),
     m_os( Os::get() ),
-    m_paused( false ),
     m_jointInd(JOINT_IND_PATH)
   {
     setEventMap(Os::get()->getEventMap(GAME_MAP));
+    sizeTo(Vec2(width,height));
+    transparent(true); //don't clear
+    m_greedyMouse = true; //get mouse clicks outside the window!
+
     m_jointInd.scale( 12.0f / (float32)m_jointInd.bbox().width() );
     //m_jointInd.simplify( 2.0f );
     m_jointInd.makeRelative();
-    configureScreenTransform( m_window.width(), m_window.height() );
+    configureScreenTransform( width, height );
     m_levels = levels;
     gotoLevel(0);
+    //add( new Button("O",Event::OPTION), Rect(800-32,0,32,32) );
   }
 
 
+  const char* name() {return "Game";}
+
   void gotoLevel( int level, bool replay=false )
   {
     bool ok = false;
+    m_replaying = replay;
 
     if ( replay ) {
       // reset scene, delete user strokes, but retain log
@@ -144,10 +114,13 @@ public:
       //m_window.setSubName( file );
       if ( m_edit ) {
 	m_scene.protect(0);
+      } else {
       }
       m_refresh = true;
       m_level = level;
-      m_stats.reset(SDL_GetTicks());
+      if (!m_replaying) {
+	m_stats.reset(SDL_GetTicks());
+      }
     }
   }
 
@@ -159,14 +132,16 @@ public:
       p = file;
     } else {
       p = Config::userDataDir() + Os::pathSep + "L99_saved.nph";
+      file = "L99_saved.nph";
     }
     if ( m_scene.save( p ) ) {
       m_levels->addPath( p.c_str() );
       int l = m_levels->findLevel( p.c_str() );
       if ( l >= 0 ) {
 	m_level = l;
-	m_window.setSubName( p.c_str() );
+	//m_window.setSubName( p.c_str() );
       }
+      showMessage(std::string("saved to<P>")+file);
       return true;
     }
     return false;
@@ -195,6 +170,43 @@ public:
     return false;
   }
 
+  void saveDemo()
+  {
+    std::string demoname = m_levels->levelName(m_level,false);
+    size_t sep = demoname.rfind(Os::pathSep);
+    if (sep != std::string::npos) {
+      demoname = demoname.substr(sep);
+    }
+    if (demoname.substr(demoname.length()-4) == ".nph") {
+      demoname.resize(demoname.length()-4);
+    }
+    int c = m_levels->collectionFromLevel(m_level);
+    std::string path = Config::userDataDir() + Os::pathSep
+      + "Recordings" + Os::pathSep
+      + m_levels->collectionName(c,false);
+    if (path.substr(path.length()-4) == ".npz") {
+      path.resize(path.length()-4);
+    }
+    OS->ensurePath(path);
+    demoname = path + Os::pathSep + demoname + ".npd";
+    fprintf(stderr,"saving demo of level %d to %s\n",
+	    m_level, demoname.c_str());
+    m_scene.save(demoname, true);
+  }
+
+  void clickMode(int cm)
+  {
+    if (cm != m_clickMode) {
+      fprintf(stderr,"clickMode=%d!\n",cm);
+      m_clickMode = cm;
+      switch (cm) {
+      case 1: setEventMap(Os::get()->getEventMap(GAME_MOVE_MAP)); break;
+      case 2: setEventMap(Os::get()->getEventMap(GAME_ERASE_MAP)); break;
+      default: setEventMap(Os::get()->getEventMap(GAME_MAP)); break;
+      }
+    }
+  }
+
   void setTool( int t )
   {
     m_colour = t;
@@ -207,33 +219,23 @@ public:
 
   void showMessage( const std::string& msg )
   {
-    //todo
     printf("showMessage \"%s\"\n",msg.c_str());
-  }
-
-  void showOverlay( Overlay* o )
-  {
-    parent()->add( o );
-    o->onShow();
-  }
-
-  void hideOverlay( Overlay* o )
-  {
-    parent()->remove( o );
-    o->onHide();
-    m_refresh = true;
+    add( new MessageBox(msg) );
   }
 
   void togglePause()
   {
     if ( !m_paused ) {
-      if ( !m_pauseOverlay ) {
-	m_pauseOverlay = createIconOverlay( *this, "pause.png", 50, 50 );
+      if ( !m_pauseLabel ) {
+      	m_pauseLabel = new Button("PAUSED",Event::PAUSE);
+	m_pauseLabel->setBg(0xff0000);
+	m_pauseLabel->setFg(0x000000);
       }
-      showOverlay( m_pauseOverlay );
+      add( m_pauseLabel, Rect(0,0,100,32) );
       m_paused = true;
     } else {
-      hideOverlay( m_pauseOverlay );
+      remove( m_pauseLabel );
+      m_pauseLabel = NULL;
       m_paused = false;
     }
   }
@@ -247,14 +249,17 @@ public:
   {
     if ( m_edit != doEdit ) {
       m_edit = doEdit;
-      if ( m_edit ) {
-	if ( !m_editOverlay ) {
-	  m_editOverlay = createEditOverlay(*this);
-	}
-	showOverlay( m_editOverlay );
+      if ( m_edit ) {	
+	if ( !m_editLabel ) {
+ 	  m_editLabel = new Button("EDIT",Event::OPTION);
+	  m_editLabel->setBg(0xff0000);
+	  m_editLabel->setFg(0x000000);
+ 	}
+	add(m_editLabel, Rect(0,0,100,32));
 	m_scene.protect(0);
       } else {
-	hideOverlay( m_editOverlay );
+	remove(m_editLabel);
+	m_editLabel = NULL;
 	m_strokeFixed = false;
 	m_strokeSleep = false;
 	m_strokeDecor = false;
@@ -279,7 +284,7 @@ public:
   virtual bool isDirty()
   {
     //TODO this can be a bit heavyweight
-    return !dirtyArea().isEmpty();
+    return Container::isDirty() || !dirtyArea().isEmpty();
   }
 
   virtual Rect dirtyArea() 
@@ -287,7 +292,6 @@ public:
     //todo include dirt  for old joint candidates
     m_jointCandidates.empty();
     if ( m_refresh  ) {
-      m_refresh = false;
       if ( m_createStroke ) {
 	//this messes up dirty calc so do _after_ dirty area eval.
 	m_scene.getJointCandidates( m_createStroke, m_jointCandidates );
@@ -305,34 +309,49 @@ public:
 	}
       }
       r.grow(8);
+      r.expand(Container::dirtyArea());
       return r;
     }
   }
 
+  void remove( Widget* w )
+  {
+    if (w==m_completedDialog) {
+      m_completedDialog = NULL;
+    }
+    Container::remove(w);
+  }
+
   virtual void onTick( int tick ) 
   {
     m_scene.step( isPaused() );
 
-    if ( m_isCompleted && m_edit ) {
-      hideOverlay( m_completedOverlay );
+    if ( m_isCompleted && m_completedDialog && m_edit ) {
+      remove( m_completedDialog );
+      m_completedDialog = NULL;
       m_isCompleted = false;
     }
     if ( m_scene.isCompleted() != m_isCompleted && !m_edit ) {
       m_isCompleted = m_scene.isCompleted();
       if ( m_isCompleted ) {
-	m_stats.endTime = SDL_GetTicks();
+	if (m_stats.endTime==0) {
+	  //don't overwrite time after replay
+	  m_stats.endTime = SDL_GetTicks();
+	}
 	fprintf(stderr,"STATS:\ttime=%dms\n\t"
 		"strokes=%d (%d paused, %d undone)\n",
 		m_stats.endTime-m_stats.startTime, m_stats.strokeCount,
 		m_stats.pausedStrokes, m_stats.undoCount);
-	showOverlay( m_completedOverlay );
-      } else {
-	hideOverlay( m_completedOverlay );
+	m_completedDialog = createNextLevelDialog(this);
+	add( m_completedDialog );
+	saveDemo();
+      } else if (m_completedDialog) {
+	remove( m_completedDialog );
+	m_completedDialog = NULL;
       }
     }
 
     if ( m_os ) {
-      m_os->poll();      
       for ( char *f = m_os->getLaunchFile(); f; f=m_os->getLaunchFile() ) {
 	if ( strstr(f,".npz") ) {
 	  //m_levels->empty();
@@ -341,15 +360,17 @@ public:
 	int l = m_levels->findLevel( f );
 	if ( l >= 0 ) {
 	  gotoLevel( l );
-	  m_window.raise();
+	  //m_window.raise();
 	}
       }    
     }  
+    Container::onTick(tick);
   }
 
   virtual void draw( Canvas& screen, const Rect& area )
   {
     static int drawCount = 0 ;
+    m_refresh = false;
     m_scene.draw( screen, area );
     if ( m_jointCandidates.size() ) {
       float32 rot = (float32)(drawCount&127) / 128.0f;
@@ -365,29 +386,90 @@ public:
     if ( m_fade ) {
       screen.fade( area );
     }
+    Container::draw(screen,area);
   }
 
-  virtual bool handleEvent( Event& ev )
+  virtual bool processEvent( SDL_Event& ev )
+  {
+    Event opt1Event(Event::OPTION,1);
+    Event opt2Event(Event::OPTION,2);
+    if (ev.type==SDL_MOUSEBUTTONDOWN) {
+      if (ev.button.x < 10
+	  && dispatchEvent(opt1Event)) {
+	return true;
+      } else if (ev.button.x > SCREEN_WIDTH-10
+		 && dispatchEvent(opt2Event)) {
+	return true;
+      }
+    }
+    return Container::processEvent(ev);
+  }
+
+  virtual bool onEvent( Event& ev )
   {
     bool used = true;
     switch (ev.code) {
+    case Event::MENU:
+      remove( m_completedDialog );
+      add( createMainMenu(this) );
+      break;
     case Event::PAUSE:
+      fprintf(stderr,"Game pause!\n");
       togglePause();
       break;
     case Event::UNDO:
-      if ( m_createStroke ) {
-	m_scene.deleteStroke( m_createStroke );
-	m_createStroke = NULL;
-      } else if ( m_scene.deleteStroke( m_scene.strokes().at(m_scene.strokes().size()-1) ) ) {
-	m_stats.undoCount++;
+      if ( !m_replaying ) {
+	if ( m_createStroke ) {
+	  m_scene.deleteStroke( m_createStroke );
+	  m_createStroke = NULL;
+	} else if ( m_scene.deleteStroke( m_scene.strokes().at(m_scene.strokes().size()-1) ) ) {
+	  m_stats.undoCount++;
+	}
       }
       m_refresh = true;
       break;
     case Event::SAVE:
       save();
       break;
-    case Event::MENU:
-      showOverlay( createMenuOverlay( *this ) );
+    case Event::CANCEL:
+      if ( m_edit ) {
+	edit(false);
+      }
+      break;
+    case Event::OPTION:
+      remove( m_options );
+      if (ev.x == 1) {
+	//edit menu
+	m_options = createEditOpts(this);
+      } else if (ev.x == 2) {
+	//play menu
+	m_options = createPlayOpts(this);
+      }
+      if (m_options) {
+	add( m_options );
+      }
+      break;
+    case Event::SELECT:
+      switch (ev.x) {
+      case 1:
+	switch (ev.y) {
+	case -1:
+	  add( createColourDialog(this, NUM_BRUSHES, brushColours) ); 
+	  break;
+	default:
+	  fprintf(stderr,"SetTool %d\n",ev.y);
+	  setTool(ev.y);
+	  break;
+	}
+	break;
+      case 2:
+	switch (ev.y) {
+	case -1:
+	  add( createToolDialog(this) );
+	  break;
+	}	    
+	break;
+      }
       break;
     case Event::EDIT:
       edit( !m_edit );
@@ -405,8 +487,8 @@ public:
       gotoLevel( m_level, true );
       break;
     case Event::DRAWBEGIN:
-      if ( !m_createStroke ) {
-	int attrib;
+      if ( !m_replaying && !m_createStroke ) {
+	int attrib = 0;
 	if ( m_strokeFixed ) attrib |= ATTRIB_GROUND;
 	if ( m_strokeSleep ) attrib |= ATTRIB_SLEEPING;
 	if ( m_strokeDecor ) attrib |= ATTRIB_DECOR;
@@ -433,13 +515,15 @@ public:
       }
       break;
     case Event::MOVEBEGIN:
-      if ( !m_moveStroke ) {
+      fprintf(stderr,"MOVING!\n");
+      if ( !m_replaying && !m_moveStroke ) {
 	m_moveStroke = m_scene.strokeAtPoint( mousePoint(ev),
 					      SELECT_TOLERANCE );
       }
       break;
     case Event::MOVEMORE:
       if ( m_moveStroke ) {
+	fprintf(stderr,"MOVING MORE!\n");
 	m_scene.moveStroke( m_moveStroke, mousePoint(ev) );
       }
       break;
@@ -447,29 +531,17 @@ public:
       m_moveStroke = NULL;
       break;
     case Event::DELETE:
+      fprintf(stderr,"DELETEING!\n");
       m_scene.deleteStroke( m_scene.strokeAtPoint( mousePoint(ev),
 						   SELECT_TOLERANCE ) );
       m_refresh = true;
       break;
     default:
-      used = false;
+      used = Container::onEvent(ev);
     }
     return used;
   }
 
-//   virtual bool handleEvent( SDL_Event& ev )
-//   {
-//     bool isReplay = m_scene.replay()->isRunning();
-
-//     if (handleModEvent(ev)
-// 	|| handleGameEvent(ev)
-// 	|| (!isReplay && (handleEditEvent(ev)
-// 			  || handlePlayEvent(ev)))) {
-//       return true;
-//     }
-//     return false;
-//   }
-
 };
 
 
