@@ -167,7 +167,7 @@ public:
     if ( m_rawPath.size() < 2 ) {
       throw "invalid stroke def";
     }
-    fprintf(stderr,"created stroke with %d points\n",m_rawPath.size());
+    //fprintf(stderr,"created stroke with %d points\n",m_rawPath.size());
     m_origin = m_rawPath.point(0);
     m_rawPath.translate( -m_origin );
     setAttribute( ATTRIB_DUMMY );
@@ -201,9 +201,10 @@ public:
       if ( m_colour==brushColours[i] )  s<<i;
     }
     s << ":";
-    transform();
-    for ( int i=0; i<m_xformedPath.size(); i++ ) {
-      const Vec2& p = m_xformedPath.point(i);
+    Path opath = m_rawPath;
+    opath.translate(m_origin);
+    for ( int i=0; i<opath.size(); i++ ) {
+      const Vec2& p = opath.point(i);
       s <<' '<< p.x << ',' << p.y; 
     }
     s << std::endl;
@@ -368,6 +369,7 @@ public:
       m_body->SetXForm( pw, m_body->GetAngle() );
     }
     m_origin = p;
+    m_drawn = false;
   }
 
   b2Body* body() { return m_body; }
@@ -498,7 +500,7 @@ private:
       m_xformedPath.translate( m_origin );
       worldToScreen.transform( m_xformedPath, m_screenPath );
       m_screenBbox = m_screenPath.bbox();      
-      return false;
+      return !hasAttribute(ATTRIB_DECOR);
     }
     return true;
   }
@@ -527,7 +529,8 @@ Scene::Scene( bool noWorld )
     m_protect( 0 ),
     m_gravity(0.0f, GRAVITY_ACCELf*PIXELS_PER_METREf/GRAVITY_FUDGEf),
     m_dynamicGravity(false),
-    m_accelerometer(Os::get()->getAccelerometer())
+    m_accelerometer(Os::get()->getAccelerometer()),
+    m_dirtyArea(false)
 {
   if ( !noWorld ) {
     b2AABB worldAABB;
@@ -551,6 +554,8 @@ Scene::~Scene()
 
 Stroke* Scene::newStroke( const Path& p, int colour, int attribs ) {
   Stroke *s = new Stroke(p);
+  s->setAttribute( (Attribute)attribs );
+
   switch ( colour ) {
   case 0: s->setAttribute( ATTRIB_TOKEN ); break;
   case 1: s->setAttribute( ATTRIB_GOAL ); break;
@@ -664,14 +669,21 @@ void Scene::step( bool isPaused )
   isPaused |= m_player.tick();
 
   if ( !isPaused ) {
-    if (m_dynamicGravity && m_accelerometer) {
+    if (m_accelerometer && m_dynamicGravity) {
       float32 gx, gy, gz;
       if ( m_accelerometer->poll( gx, gy, gz ) ) {
-	//fprintf(stderr,"dynamic grav = %f,%f\n", gx, gy );
-	const float32 factor = GRAVITY_ACCELf*PIXELS_PER_METREf/GRAVITY_FUDGEf;
-	b2Vec2 accel( m_gravity.x + gx*factor, 
-		      m_gravity.y + gy*factor );
-	m_world->SetGravity( accel );
+	
+	if (m_dynamicGravity || gx*gx+gy*gy > 1.2*1.2)  {
+	  //fprintf(stderr,"dynamic grav = %f,%f\n", gx, gy );
+	  const float32 factor = GRAVITY_ACCELf*PIXELS_PER_METREf/GRAVITY_FUDGEf;
+	  m_currentGravity = b2Vec2( m_gravity.x + gx*factor, 
+				     m_gravity.y + gy*factor );
+	  m_world->SetGravity( m_currentGravity );
+	} else if (!(m_currentGravity == m_gravity)) {
+	  m_currentGravity += m_gravity;
+	  m_currentGravity *= 0.5;
+	  m_world->SetGravity( m_currentGravity );
+	}
 	//TODO record gravity
       }
     }
@@ -692,6 +704,7 @@ void Scene::step( bool isPaused )
 	activate( m_strokes[i] );	  
       }
     }
+    calcDirtyArea();
   }
 }
 
@@ -726,6 +739,11 @@ bool Scene::isCompleted()
 }
 
 Rect Scene::dirtyArea()
+{
+  return m_dirtyArea;
+}
+
+void Scene::calcDirtyArea()
 {
   Rect r(false);
   for ( int i=0; i<m_strokes.size(); i++ ) {
@@ -744,7 +762,8 @@ Rect Scene::dirtyArea()
     // expand to allow for thick lines
     r.grow(1);
   }
-  return r;
+  //fprintf(stderr,"scene dirty %d,%d-%d,%d!\n",r.tl.x,r.tl.y,r.br.x,r.br.y);
+  m_dirtyArea = r;
 }
 void Scene::draw( Canvas& canvas, const Rect& area )
 {
@@ -817,15 +836,14 @@ void Scene::clear()
 
 void Scene::setGravity( const b2Vec2& g )
 {
-  m_gravity = g;
-  m_world->SetGravity( m_gravity );
+  m_gravity = m_currentGravity = g;
+  if (m_world) {
+    m_world->SetGravity( m_gravity );
+  }
 }
 
 void Scene::setGravity( const std::string& s )
 {
-  m_dynamicGravity = false;
-  setGravity( b2Vec2(0.0f, GRAVITY_ACCELf*PIXELS_PER_METREf/GRAVITY_FUDGEf) );
-
   for (int i=0; i<s.find(':'); i++) {
     switch (s[i]) {
     case 'd': m_dynamicGravity = true; break;
@@ -861,8 +879,10 @@ bool Scene::load( const std::string& file )
 bool Scene::load( std::istream& in )
 {
   clear();
+  m_dynamicGravity = false;
+  setGravity( b2Vec2(0.0f, GRAVITY_ACCELf*PIXELS_PER_METREf/GRAVITY_FUDGEf) );
   if ( g_bgImage==NULL ) {
-    g_bgImage = new Image("paper.png");
+    g_bgImage = new Image("paper.jpg");
     g_bgImage->scale( SCREEN_WIDTH, SCREEN_HEIGHT );
   }
   m_bgImage = g_bgImage;
@@ -915,9 +935,6 @@ void Scene::protect( int n )
 bool Scene::save( const std::string& file, bool saveLog )
 {
   printf("saving to %s\n",file.c_str());
-  if ( saveLog ) {      
-    reset();
-  }
   std::ofstream o( file.c_str(), std::ios::out );
   if ( o.is_open() ) {
     o << "Title: "<<m_title<<std::endl;
